import java.util.ArrayList;
import java.util.Random;

public class SAPI {
	public static class Dungeon {
		private static ArrayList<Pair<Float,DungeonLoot>> loot = new ArrayList<Pair<Float,DungeonLoot>>();
		private static ArrayList<Pair<Float,String>> mobs = new ArrayList<Pair<Float,String>>();

		public static void addDungeonLoot(DungeonLoot dl) {
			addDungeonLoot(dl,1f);
		}
		public static void addDungeonLoot(DungeonLoot dl, float commonness) {
			loot.add(new Pair<Float,DungeonLoot>(commonness,dl));
		}
		public static void addMob(String mob) {
			addMob(mob,1f);
		}
		public static void addMob(String mob, float commonness) {
			mobs.add(new Pair<Float,String>(commonness,mob));
		}
		private static float lootGetMaxRandom() {
			float ret = 0f;
			for (Pair<Float,DungeonLoot> pair : loot) ret += pair.get1();
			return ret;
		}
		private static DungeonLoot lootGetFromRandomFloat(float random) {
			for (Pair<Float,DungeonLoot> pair : loot) {
				if (random < pair.get1()) return pair.get2();
				random -= pair.get1();
			}
			return null;
		}
		public static DungeonLoot lootGet(Random rnd) {
			return lootGetFromRandomFloat(rnd.nextFloat() * lootGetMaxRandom());
		}
		private static float mobGetMaxRandom() {
			float ret = 0f;
			for (Pair<Float,String> pair : mobs) ret += pair.get1();
			return ret;
		}
		private static String mobGetFromRandomFloat(float random) {
			for (Pair<Float,String> pair : mobs) {
				if (random < pair.get1()) return pair.get2();
				random -= pair.get1();
			}
			return null;
		}
		public static String mobGet(Random rnd) {
			return mobGetFromRandomFloat(rnd.nextFloat() * mobGetMaxRandom());
		}
	}

	public static class Achievements {
		private static ArrayList<AchievementPage> pages = new ArrayList<AchievementPage>();
		private static ArrayList<Integer> hidden = new ArrayList<Integer>();
		private static int current = 0;
		protected static AchievementPage defaultPage = new AchievementPage();

		public static void addPage(AchievementPage page) {
			pages.add(page);
		}
		public static AchievementPage getPage(aca achievement) {
			if (achievement == null) return null;
			for (AchievementPage page : pages) if (page.list.contains(achievement.a)) return page;
			return defaultPage;
		}
		public static void hide(aca... achievements) {
			for (aca a : achievements) hidden.add(a.a);
		}
		public static void unhide(aca... achievements) {
			for (aca a : achievements) hidden.remove((Object)a.a);
		}
		public static boolean isHidden(aca achievement) {
			for (int a : hidden) if (achievement.a == a) return true;
			return false;
		}
		public static ArrayList<AchievementPage> getPages() {
			return new ArrayList<AchievementPage>(pages);
		}
		public static AchievementPage current() {
			return pages.get(current);
		}
		public static void setPage(AchievementPage page) {
			for (int i = 0; i < pages.size(); i++) if (pages.get(i).equals(page)) {
				current = i;
				return;
			}
		}
	}

	public static class LockDifficulty {
		private static ArrayList<ILockDifficulty> mods = new ArrayList<ILockDifficulty>();

		public static void add(ILockDifficulty mod) {
			mods.add(mod);
		}
		public static String listLockingMods() {
			String s = "";
			if (mods.isEmpty()) return s;
			for (ILockDifficulty mod : mods) {
				if (!mod.iLockShouldLock()) continue;
				if (!s.isEmpty()) s += ",";
				s += mod.iLockModName();
			}
			return s;
		}
	}

	public static class NBT {
		private static ArrayList<INBT> nbt = new ArrayList<INBT>();

		public static void add(INBT inbt) {
			nbt.add(inbt);
		}
		public static ArrayList<INBT> getList() {
			return new ArrayList<INBT>(nbt);
		}
	}

	public static class Reach {
		private static ArrayList<IReach> reach = new ArrayList<IReach>();

		public static void add(IReach ireach) {
			reach.add(ireach);
		}
		public static ArrayList<IReach> getList() {
			return new ArrayList<IReach>(reach);
		}
		public static float getReach(boolean entity) {
			yq hand = ModLoader.getMinecraftInstance().h.ae();
			for (IReach r : reach) if (r.iReachMatches(entity,hand)) return r.iReachGet(entity,hand);
			return entity ? 3f : 4f;
		}
	}

	public static class BlockSet {
		private static ArrayList<IBlockSet> blockset = new ArrayList<IBlockSet>();

		public static void add(IBlockSet iblockset) {
			blockset.add(iblockset);
		}
		public static ArrayList<IBlockSet> getList() {
			return new ArrayList<IBlockSet>(blockset);
		}
		public static IBlockSet getIntercept(vq world, int x, int y, int z, int blockID) {
			for (IBlockSet i : blockset) if (i.iBlockMatches(world,x,y,z,blockID)) return i;
			return null;
		}
	}

	public static class Harvest {
		private static ArrayList<IHarvest> harvest = new ArrayList<IHarvest>();

		public static void add(IHarvest iharvest) {
			harvest.add(iharvest);
		}
		public static ArrayList<IHarvest> getList() {
			return new ArrayList<IHarvest>(harvest);
		}
		public static boolean intercept(vq world, xb player, int x, int y, int z, int blockID, int metadata) {
			for (IHarvest i : harvest) if (i.iHarvest(world,player,x,y,z,blockID,metadata)) return true;
			return false;
		}
	}
	
	public static class ItemBreak {
		private static ArrayList<IItemBreak> items = new ArrayList<IItemBreak>();
		
		public static void add(IItemBreak iitembreak) {
			items.add(iitembreak);
		}
		public static ArrayList<IItemBreak> getList() {
			return new ArrayList<IItemBreak>(items);
		}
		public static void itemBroke(xb player, yq stack) {
			for (IItemBreak i : items) if (i.iItemBroke(player,stack)) return;
		}
	}
	
	public static class Pair<A,B> {
		private A first;
		private B second;

		public Pair(A first, B second) {
			super();
			this.first = first;
			this.second = second;
		}

		public boolean equals(Object instance) {
			if (instance instanceof Pair<?,?>) {
				Pair<?,?> other = (Pair<?,?>)instance;
				boolean check1 = false, check2 = false;
				
				if (first != null) check1 = first.equals(other.first);
				if (!check1 && other.first != null) check1 = other.first.equals(first);
				if (second != null) check2 = second.equals(other.second);
				if (!check2 && other.second != null) check2 = other.second.equals(second);
				
				return check1 && check2;
			}
			return false;
		}
		public String toString() { return "("+first+"|"+second+")";}

		public A get1() {return first;}
		public void set1(A first) {this.first = first;}
		public B get2() {return second;}
		public void set2(B second) {this.second = second;}
	}
}